/*
* Copyright © 2020, Concordant and contributors.
*
* Permission is hereby granted, free of charge, to any person obtaining a copy of this software and
* associated documentation files (the "Software"), to deal in the Software without restriction,
* including without limitation the rights to use, copy, modify, merge, publish, distribute,
* sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in all copies or
* substantial portions of the Software.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT
* NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
* NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
* DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/

package client

import client.utils.CObjectUId
import client.utils.CollectionUId
import client.utils.NotificationHandler

class Collection {

    /**
    * The collection unique id
    */
    private val id: CollectionUId

    /**
    * Is the collection open in read only mode
    */
    private val readOnly: Boolean

    constructor(cid: CollectionUId, readOnly: Boolean) {
        this.id = cid
        this.readOnly = readOnly
    }

    // c_open_read|write<T>
    fun <T> open(oid: String, readOnly: Boolean, handler: NotificationHandler<T>): T {
        if (this.readOnly && !readOnly) throw RuntimeException("Collection has been opened in read-only mode.")

        val oid = CObjectUId<T>(this.id, oid)
        return CObject<T>(oid, readOnly) as T
    }

    // c_close_collection
    fun close() {
    }
}
